@@ -2,6 +2,7 @@ import base64
 import datetime
 import json
 import os
+import time
 from typing import Optional
 from urllib.parse import urlparse
 
@@ -20,11 +21,14 @@ from bpkio_api.endpoints import (
     TranscodingProfilesApi,
     UsersApi,
 )
+from bpkio_api.endpoints.login import LoginApi
 from bpkio_api.exceptions import (
     BroadpeakIoApiError,
+    ExpiredApiKeyFormat,
     InvalidApiKeyFormat,
     InvalidEndpointError,
     InvalidTenantError,
+    MissingApiKeyError,
 )
 from bpkio_api.helpers.recorder import SessionRecorder
 from bpkio_api.mappings import model_to_endpoint
@@ -101,11 +105,15 @@ class BroadpeakIoApi(BpkioSdkConsumer):
                 "or have configured a default tenant"
             )
 
+        # Check if API key is missing
+        if not self._api_key:
+            raise MissingApiKeyError()
+
         base_url = f"https://{self._fqdn}/v1/"
 
         super().__init__(base_url, auth=BearerToken(self._api_key), **kwargs)
 
-        tenant_id = self.parse_api_key().get("tenantId")
+        tenant_id = self.get_tenant_id()
         if use_cache:
             init_cache(self.fqdn, tenant_id)
 
@@ -124,6 +132,7 @@ class BroadpeakIoApi(BpkioSdkConsumer):
         self.categories = CategoriesApi(
             base_url, auth=BearerToken(self._api_key), **kwargs
         )
+        self.login = LoginApi(base_url, auth=None, **kwargs)
 
         # Optional endpoints
         try:
@@ -132,8 +141,11 @@ class BroadpeakIoApi(BpkioSdkConsumer):
             add_admin_endpoints(
                 self, base_url, auth=BearerToken(self._api_key), **kwargs
             )
+        except ImportError:
+            # Admin module is optional and may not be installed
+            pass
         except Exception:
-            # print(e)
+            # Other errors are silently ignored for optional admin endpoints
             pass
 
     @staticmethod
@@ -153,7 +165,15 @@ class BroadpeakIoApi(BpkioSdkConsumer):
             s = base64.b64decode(base64_bytes).decode("utf-8")
             payload = json.loads(s)
             # self.logger.debug("API Key payload: " + str(payload))
+
+            # check expiration date
+            if payload.get("exp") and payload.get("exp") < time.time():
+                raise ExpiredApiKeyFormat(reason="API key expired")
+
             return payload
+
+        except ExpiredApiKeyFormat as e:
+            raise e
         except Exception as e:
             raise InvalidApiKeyFormat(reason=e)
 
@@ -167,12 +187,35 @@ class BroadpeakIoApi(BpkioSdkConsumer):
         try:
             BroadpeakIoApi._parse_api_key(string)
             return True
-        except:
+        except Exception:
             return False
 
+    @staticmethod
+    def is_expired_api_key(string: str):
+        """Checks if the API Key is expired"""
+        try:
+            BroadpeakIoApi._parse_api_key(string)
+            return False
+        except ExpiredApiKeyFormat:
+            return True
+
     def get_tenant_id(self) -> int:
         """Returns the tenant ID from the API Key"""
-        return self.parse_api_key().get("tenantId")
+        apikey_info = self.parse_api_key()
+        tenant_id = apikey_info.get("tenantId")
+        if not tenant_id:
+            # try to get the tenant ID from the API key payload,
+            # by getting the value from any key ending with /tenant_id (allowing for multi-platform support)
+            tenant_id = next(
+                (
+                    value
+                    for key, value in apikey_info.items()
+                    if key.endswith("/tenant_id")
+                ),
+                None,
+            )
+
+        return tenant_id
 
     @SessionRecorder.do_not_record
     def get_self_tenant(self) -> Tenant:
@@ -217,6 +260,9 @@ class BroadpeakIoApi(BpkioSdkConsumer):
         if fqdn.startswith("app"):
             fqdn = "api" + fqdn[3:]
 
+        if fqdn.startswith("login"):
+            fqdn = "api" + fqdn[5:]
+
         return fqdn
 
     @staticmethod
