@@ -1,4 +1,4 @@
-from typing import Any, List, Tuple
+from typing import Any, Callable, List, Tuple
 from urllib.parse import urlparse, urlunparse
 
 from uplink import Body, Query, delete, get, json, post, put, response_handler, returns
@@ -13,11 +13,10 @@ from bpkio_api.helpers.list import (
 )
 from bpkio_api.helpers.objects import find_duplicates_of
 from bpkio_api.helpers.search import SearchMethod, search_array_with_filters
+from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
 from bpkio_api.models import Sources as S
 from bpkio_api.response_handler import extract_count, postprocess_response
 
-from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
-
 
 @response_handler(postprocess_response)
 class SourcesApi(BpkioSdkConsumer):
@@ -72,7 +71,11 @@ class SourcesApi(BpkioSdkConsumer):
     # === Helpers ===
 
     @cache_api_results("list_sources", key_suffix_from_arg="sparse")
-    def list(self, sparse: bool = True):
+    def list(
+        self,
+        sparse: bool = True,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ):
         """List all sources
 
         Args:
@@ -81,12 +84,16 @@ class SourcesApi(BpkioSdkConsumer):
         Returns:
             List[S.SourceSparse | S.AssetSource | S.LiveSource | S.AdServerSource | S.AssetCatalogSource | S.SlateSource | S.OriginSource]: List of sources
         """
-        sparse_sources = get_all_with_pagination(self._get_page, count_fn=self.count)
+        sparse_sources = get_all_with_pagination(
+            self._get_page, count_fn=self.count, progress_hook=progress_hook
+        )
         if sparse:
             return sparse_sources
         else:
             return collect_from_sparse_resources(
-                sparse_resources=sparse_sources, get_fn=self._get_by_type
+                sparse_resources=sparse_sources,
+                get_fn=self._get_by_type,
+                progress_hook=progress_hook,
             )
 
     def _get_by_type(
@@ -123,6 +130,8 @@ class SourcesApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        sparse: bool = True,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[S.SourceSparse]:
         """Searches the list of sources for those matching a particular filter query
 
@@ -148,7 +157,7 @@ class SourcesApi(BpkioSdkConsumer):
         if not filters:
             filters = [(value, field, method)]
 
-        sources = self.list()
+        sources = self.list(sparse=sparse, progress_hook=progress_hook)
         return search_array_with_filters(sources, filters=filters)
 
     def retrieve(
@@ -370,13 +379,25 @@ class AssetSourcesApi(BpkioSdkConsumer):
         )
 
     @cache_api_results("list_assets")
-    def list(self) -> List[S.AssetSource]:
+    def list(
+        self,
+        sparse: bool = False,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[S.AssetSource]:
         """List all Asset sources"""
         sparse_sources = self.parent_api.search(
-            S.SourceType.ASSET, field="type", method=SearchMethod.STRICT
+            S.SourceType.ASSET,
+            field="type",
+            method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=sparse,
         )
+        if sparse:
+            return sparse_sources
         return collect_from_ids(
-            ids=[src.id for src in sparse_sources], get_fn=self.retrieve
+            ids=[src.id for src in sparse_sources],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
 
     def search(
@@ -385,12 +406,13 @@ class AssetSourcesApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[S.AssetSource]:
         """Search the list of Asset sources for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        sources = self.list()
+        sources = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(sources, filters=filters)
 
 
@@ -433,13 +455,25 @@ class LiveSourcesApi(BpkioSdkConsumer):
         )
 
     @cache_api_results("list_live")
-    def list(self) -> List[S.LiveSource]:
+    def list(
+        self,
+        sparse: bool = False,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[S.LiveSource]:
         """List all Live sources"""
         sparse_sources = self.parent_api.search(
-            S.SourceType.LIVE, field="type", method=SearchMethod.STRICT
+            S.SourceType.LIVE,
+            field="type",
+            method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=True,
         )
+        if sparse:
+            return sparse_sources
         return collect_from_ids(
-            ids=[src.id for src in sparse_sources], get_fn=self.retrieve
+            ids=[src.id for src in sparse_sources],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
 
     def search(
@@ -448,12 +482,13 @@ class LiveSourcesApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[S.LiveSource]:
         """Search the list of Live sources for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        sources = self.list()
+        sources = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(sources, filters=filters)
 
 
@@ -498,13 +533,25 @@ class AssetCatalogSourcesApi(BpkioSdkConsumer):
         )
 
     @cache_api_results("list_assetcatalogs")
-    def list(self) -> List[S.AssetCatalogSource]:
+    def list(
+        self,
+        sparse: bool = False,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[S.AssetCatalogSource]:
         """List all Asset Catalog sources"""
         sparse_sources = self.parent_api.search(
-            S.SourceType.ASSET_CATALOG, field="type", method=SearchMethod.STRICT
+            S.SourceType.ASSET_CATALOG,
+            field="type",
+            method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=sparse,
         )
+        if sparse:
+            return sparse_sources
         return collect_from_ids(
-            ids=[src.id for src in sparse_sources], get_fn=self.retrieve
+            ids=[src.id for src in sparse_sources],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
 
     def search(
@@ -513,12 +560,13 @@ class AssetCatalogSourcesApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[S.AssetCatalogSource]:
         """Search the list of Asset Catalog sources for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        sources = self.list()
+        sources = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(sources, filters=filters)
 
 
@@ -601,13 +649,25 @@ class AdServerSourcesApi(BpkioSdkConsumer):
         return self.create(ad_server_source)
 
     @cache_api_results("list_adservers")
-    def list(self) -> List[S.AdServerSource]:
+    def list(
+        self,
+        sparse: bool = False,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[S.AdServerSource]:
         """List all Ad Server sources"""
         sparse_sources = self.parent_api.search(
-            S.SourceType.AD_SERVER, field="type", method=SearchMethod.STRICT
+            S.SourceType.AD_SERVER,
+            field="type",
+            method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=sparse,
         )
+        if sparse:
+            return sparse_sources
         return collect_from_ids(
-            ids=[src.id for src in sparse_sources], get_fn=self.retrieve
+            ids=[src.id for src in sparse_sources],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
 
     def search(
@@ -616,12 +676,13 @@ class AdServerSourcesApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[S.AdServerSource]:
         """Searches the list of Ad Server sources for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        sources = self.list()
+        sources = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(sources, filters=filters)
 
 
@@ -666,13 +727,25 @@ class SlateSourcesApi(BpkioSdkConsumer):
         )
 
     @cache_api_results("list_slates")
-    def list(self) -> List[S.SlateSource]:
+    def list(
+        self,
+        sparse: bool = False,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[S.SlateSource]:
         """List all Slate sources"""
         sparse_sources = self.parent_api.search(
-            S.SourceType.SLATE, field="type", method=SearchMethod.STRICT
+            S.SourceType.SLATE,
+            field="type",
+            method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=sparse,
         )
+        if sparse:
+            return sparse_sources
         return collect_from_ids(
-            ids=[src.id for src in sparse_sources], get_fn=self.retrieve
+            ids=[src.id for src in sparse_sources],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
 
     def search(
@@ -681,12 +754,13 @@ class SlateSourcesApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[S.SlateSource]:
         """Searches the list of Slate sources for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        sources = self.list()
+        sources = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(sources, filters=filters)
 
 
@@ -731,13 +805,25 @@ class OriginSourcesApi(BpkioSdkConsumer):
         )
 
     @cache_api_results("list_origins")
-    def list(self) -> List[S.OriginSource]:
+    def list(
+        self,
+        sparse: bool = False,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[S.OriginSource]:
         """List all Origin sources"""
         sparse_sources = self.parent_api.search(
-            S.SourceType.ORIGIN, field="type", method=SearchMethod.STRICT
+            S.SourceType.ORIGIN,
+            field="type",
+            method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=sparse,
         )
+        if sparse:
+            return sparse_sources
         return collect_from_ids(
-            ids=[src.id for src in sparse_sources], get_fn=self.retrieve
+            ids=[src.id for src in sparse_sources],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
 
     def search(
@@ -746,10 +832,11 @@ class OriginSourcesApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[S.OriginSource]:
         """Searches the list of Origin sources for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        sources = self.list()
+        sources = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(sources, filters=filters)
