@@ -1,16 +1,19 @@
 import os
 from datetime import datetime
 from enum import Enum
-from typing import List, Optional, Union
+from typing import Annotated, List, Optional, Union
 from urllib.parse import parse_qs, urlencode, urljoin, urlparse, urlunparse
 
 from media_muncher.format import MediaFormat
 from pydantic import BaseModel, ConfigDict, Field, HttpUrl
 
+from bpkio_api.helpers.type_metadata import ExpectedTypeMetadata
 from bpkio_api.models.common import BaseResource, NamedModel, PropertyMixin
 from bpkio_api.models.Sources import (
     AdServerSource,
+    AssetCatalogSource,
     AssetSource,
+    LiveSource,
     SlateSource,
     SourceSparse,
 )
@@ -136,7 +139,7 @@ class VodAdInsertionModel(BaseModel):
 
 
 class VodAdInsertionModelIn(BaseModel):
-    adServer: BaseResource
+    adServer: Annotated[BaseResource, ExpectedTypeMetadata(AdServerSource)]
 
 
 class LiveAdReplacementModel(BaseModel):
@@ -150,8 +153,10 @@ class LiveAdReplacementModel(BaseModel):
 
 
 class LiveAdReplacementModelIn(BaseModel):
-    adServer: BaseResource
-    gapFiller: Optional[BaseResource] = None
+    adServer: Annotated[BaseResource, ExpectedTypeMetadata(AdServerSource)]
+    gapFiller: Optional[
+        Annotated[BaseResource, ExpectedTypeMetadata(Union[SlateSource, AssetSource])]
+    ] = None
     frenchAddressableTV: Optional[bool] = False
     maxAdDurationExcess: float = 1
     pastAdBreakReplacement: float = 0
@@ -167,8 +172,10 @@ class AdBreakInsertionModel(BaseModel):
 
 
 class AdBreakInsertionModelIn(BaseModel):
-    adServer: BaseResource
-    gapFiller: Optional[BaseResource] = None
+    adServer: Annotated[BaseResource, ExpectedTypeMetadata(AdServerSource)]
+    gapFiller: Optional[
+        Annotated[BaseResource, ExpectedTypeMetadata(Union[SlateSource, AssetSource])]
+    ] = None
 
     model_config = ConfigDict(extra="allow")
 
@@ -182,7 +189,7 @@ class LiveAdPreRollModel(BaseModel):
 
 
 class LiveAdPreRollModelIn(BaseModel):
-    adServer: BaseResource
+    adServer: Annotated[BaseResource, ExpectedTypeMetadata(AdServerSource)]
     maxDuration: float = 360
     offset: float = 0
 
@@ -199,7 +206,9 @@ class ServerSideAdTracking(BaseModel):
 class WithCommonAdInsertionServiceFields(BaseModel):
     enableAdTranscoding: Optional[bool] = False
     serverSideAdTracking: ServerSideAdTracking
-    transcodingProfile: Optional[TranscodingProfileId] = None
+    transcodingProfile: Optional[
+        Annotated[TranscodingProfileId, ExpectedTypeMetadata(TranscodingProfile)]
+    ] = None
 
     @property
     def sub_type(self):
@@ -230,7 +239,9 @@ class WithCommonAdInsertionServiceFields(BaseModel):
 
 class AdInsertionServiceIn(ServiceIn, WithCommonAdInsertionServiceFields):
     # TODO: parse the specific sub-type of source
-    source: BaseResource
+    source: Annotated[
+        BaseResource, ExpectedTypeMetadata(Union[LiveSource, AssetCatalogSource])
+    ]
 
     vodAdInsertion: Optional[VodAdInsertionModelIn] = None
     liveAdPreRoll: Optional[LiveAdPreRollModelIn] = None
@@ -262,8 +273,8 @@ class AdInsertionService(
 
 class ContentReplacementServiceIn(ServiceIn):
     # TODO: parse the specific sub-type of source
-    source: BaseResource
-    replacement: BaseResource
+    source: Annotated[BaseResource, ExpectedTypeMetadata(LiveSource)]
+    replacement: Annotated[BaseResource, ExpectedTypeMetadata(LiveSource)]
 
     def is_live(self):
         return True
@@ -288,7 +299,9 @@ class WithCommonVirtualChannelServiceFields(BaseModel):
     enableAdTranscoding: Optional[bool] = False
     serverSideAdTracking: Optional[ServerSideAdTracking] = None
 
-    transcodingProfile: Optional[TranscodingProfileId] = None
+    transcodingProfile: Optional[
+        Annotated[TranscodingProfileId, ExpectedTypeMetadata(TranscodingProfile)]
+    ] = None
 
     def is_live(self):
         return True
@@ -296,7 +309,7 @@ class WithCommonVirtualChannelServiceFields(BaseModel):
 
 class VirtualChannelServiceIn(ServiceIn, WithCommonVirtualChannelServiceFields):
     # TODO: parse the specific sub-type of source
-    baseLive: BaseResource
+    baseLive: Annotated[BaseResource, ExpectedTypeMetadata(LiveSource)]
 
     adBreakInsertion: Optional[AdBreakInsertionModelIn] = None
 
